Package: git2r
Title: Provides Access to Git Repositories
Description: Interface to the 'libgit2' library, which is a pure C
    implementation of the 'Git' core methods. Provides access to 'Git'
    repositories to extract data and running some basic 'Git'
    commands.
Version: 0.33.0
License: GPL-2
Copyright: See COPYRIGHTS file.
URL: https://docs.ropensci.org/git2r/ (website),
        https://github.com/ropensci/git2r
BugReports: https://github.com/ropensci/git2r/issues
Maintainer: Stefan Widgren <stefan.widgren@gmail.com>
Author: See AUTHORS file.
Imports: graphics, utils
Depends: R (>= 4.0)
Suggests: getPass
Type: Package
Biarch: true
NeedsCompilation: yes
SystemRequirements: By default, git2r uses a system installation of the
        libgit2 headers and library. However, if a system installation
        is not available, builds and uses a bundled version of the
        libgit2 source. zlib headers and library. OpenSSL headers and
        library (non-macOS). LibSSH2 (optional on non-Windows) to
        enable the SSH transport.
Collate: 'blame.R' 'blob.R' 'branch.R' 'bundle_r_package.R'
        'checkout.R' 'commit.R' 'config.R' 'contributions.R'
        'credential.R' 'diff.R' 'fetch.R' 'git2r.R' 'index.R'
        'libgit2.R' 'merge.R' 'note.R' 'odb.R' 'plot.R' 'pull.R'
        'punch_card.R' 'push.R' 'reference.R' 'reflog.R' 'refspec.R'
        'remote.R' 'repository.R' 'reset.R' 'revparse.R' 'sha.R'
        'signature.R' 'stash.R' 'status.R' 'tag.R' 'time.R' 'tree.R'
        'when.R'
Encoding: UTF-8
RoxygenNote: 7.2.3
Packaged: 2023-11-26 16:10:10 UTC; stefan
Repository: CRAN
Date/Publication: 2023-11-26 16:50:05 UTC
Built: R 4.4.0; i386-portbld-midnightbsd4.0; 2026-05-12 21:45:20 UTC; unix
