# Generated by roxygen2: do not edit by hand

S3method("[",git_tree)
S3method(as.POSIXct,git_time)
S3method(as.character,git_time)
S3method(as.data.frame,git_commit)
S3method(as.data.frame,git_repository)
S3method(as.data.frame,git_tree)
S3method(as.list,git_tree)
S3method(diff,git_repository)
S3method(diff,git_tree)
S3method(format,git_blob)
S3method(format,git_commit)
S3method(format,git_merge_result)
S3method(format,git_note)
S3method(format,git_reference)
S3method(format,git_signature)
S3method(format,git_stash)
S3method(format,git_tag)
S3method(format,git_time)
S3method(head,git_repository)
S3method(length,git_blob)
S3method(length,git_diff)
S3method(length,git_tree)
S3method(lookup_commit,git_branch)
S3method(lookup_commit,git_commit)
S3method(lookup_commit,git_reference)
S3method(lookup_commit,git_tag)
S3method(merge,character)
S3method(merge,git_branch)
S3method(merge,git_repository)
S3method(plot,git_repository)
S3method(print,git_blob)
S3method(print,git_branch)
S3method(print,git_commit)
S3method(print,git_config)
S3method(print,git_diff)
S3method(print,git_merge_result)
S3method(print,git_note)
S3method(print,git_reference)
S3method(print,git_reflog)
S3method(print,git_reflog_entry)
S3method(print,git_repository)
S3method(print,git_signature)
S3method(print,git_stash)
S3method(print,git_status)
S3method(print,git_tag)
S3method(print,git_time)
S3method(print,git_tree)
S3method(sha,git_blob)
S3method(sha,git_branch)
S3method(sha,git_commit)
S3method(sha,git_fetch_head)
S3method(sha,git_merge_result)
S3method(sha,git_note)
S3method(sha,git_reference)
S3method(sha,git_reflog_entry)
S3method(sha,git_tag)
S3method(sha,git_tree)
S3method(summary,git_commit)
S3method(summary,git_diff)
S3method(summary,git_repository)
S3method(summary,git_stash)
S3method(summary,git_tag)
S3method(summary,git_tree)
export(add)
export(ahead_behind)
export(as.data.frame)
export(blame)
export(blob_create)
export(branch_create)
export(branch_delete)
export(branch_get_upstream)
export(branch_remote_name)
export(branch_remote_url)
export(branch_rename)
export(branch_set_upstream)
export(branch_target)
export(branches)
export(bundle_r_package)
export(checkout)
export(clone)
export(commit)
export(commits)
export(config)
export(content)
export(contributions)
export(cred_env)
export(cred_ssh_key)
export(cred_token)
export(cred_user_pass)
export(default_signature)
export(descendant_of)
export(diff)
export(discover_repository)
export(fetch)
export(fetch_heads)
export(git_config_files)
export(hash)
export(hashfile)
export(head)
export(in_repository)
export(index_remove_bypath)
export(init)
export(is_bare)
export(is_binary)
export(is_blob)
export(is_branch)
export(is_commit)
export(is_detached)
export(is_empty)
export(is_head)
export(is_local)
export(is_merge)
export(is_shallow)
export(is_tag)
export(is_tree)
export(last_commit)
export(libgit2_features)
export(libgit2_version)
export(lookup)
export(lookup_commit)
export(ls_tree)
export(merge)
export(merge_base)
export(note_create)
export(note_default_ref)
export(note_remove)
export(notes)
export(odb_blobs)
export(odb_objects)
export(parents)
export(pull)
export(punch_card)
export(push)
export(references)
export(reflog)
export(remote_add)
export(remote_ls)
export(remote_remove)
export(remote_rename)
export(remote_set_url)
export(remote_url)
export(remotes)
export(repository)
export(repository_head)
export(reset)
export(revparse_single)
export(rm_file)
export(sha)
export(ssh_path)
export(ssl_cert_locations)
export(stash)
export(stash_apply)
export(stash_drop)
export(stash_list)
export(stash_pop)
export(status)
export(tag)
export(tag_delete)
export(tags)
export(tree)
export(when)
export(workdir)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,symbols)
importFrom(graphics,title)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,sessionInfo)
useDynLib(git2r, .registration=TRUE)
