% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_classes.r
\docType{class}
\name{git_stash-class}
\alias{git_stash-class}
\title{S4 class to handle a git stash}
\description{
Class \code{"git_stash"}
}
\keyword{classes}
\keyword{methods}
