% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_classes.r
\docType{class}
\name{git_tag-class}
\alias{git_tag-class}
\title{S4 class to handle a git tag}
\description{
Class \code{"git_tag"}
}
\section{Slots}{

\describe{
  \item{sha}{
    The 40 character hexadecimal string of the SHA-1
  }
  \item{message}{
    The message of the tag
  }
  \item{name}{
    The name of the tag
  }
  \item{tagger}{
    The tagger (author) of the tag
  }
  \item{target}{
    The target of the tag
  }
  \item{repo}{
    The S4 class git_repository that contains the tag
  }
}
}

\keyword{classes}
\keyword{methods}
