% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4_classes.r
\docType{class}
\name{git_blob-class}
\alias{git_blob-class}
\title{S4 class to handle a git blob}
\description{
Class \code{"git_blob"}
}
\section{Slots}{

\describe{
  \item{sha}{
    The 40 character hexadecimal string of the SHA-1
  }
  \item{repo}{
    The S4 class git_repository that contains the blob
  }
}
}

\keyword{classes}
\keyword{methods}
